/**
 * @file asm.h
 * Provides assembly support C function API declarations.
 */

/**
 * @returns text_end which is bss start for initializing malloc heap.
 */
long getBSS(void);
    
/**
 * Start a cog running native code as suggested by ImageCraft.
 * @param addr is start address of native code
 * @param parm is start address of native parameters
 * @returns non-zero cogid if ok. 
 */
int cognewNative(void* addr, void* parm);

/**
 * Same as memcpy.
 * @param R0 is pointer to destination to copy
 * @param R1 is pointer to source of copy
 * @param R2 is number of words to copy
 */
void bytemove(char* R0, char* R1, int R2);

/**
 * @param R0 is pointer to destination to fill
 * @param R1 is value to fill
 * @param R2 is number of words to fill
 */
void bytefill(char*  R0, char  R1, int R2);

/**
 * @param R0 is pointer to destination to fill
 * @param R1 is value to fill
 * @param R2 is number of words to fill
 */
void wordfill(short* R0, short R1, int R2);

/**
 * @param R0 is pointer to destination to fill
 * @param R1 is value to fill
 * @param R2 is number of words to fill
 */
void longfill(long*  R0, long  R1, int R2);

/**
 * @param R0 is pointer to destination to copy
 * @param R1 is pointer to source of copy
 * @param R2 is number of words to copy
 */
void bytemove(char*  R0, char*  R1, int R2);

/**
 * @param R0 is pointer to destination to copy
 * @param R1 is pointer to source of copy
 * @param R2 is number of words to copy
 */
void wordmove(short* R0, short* R1, int R2);

/**
 * @param R0 is pointer to destination to copy
 * @param R1 is pointer to source of copy
 * @param R2 is number of words to copy
 */
void longmove(long*  R0, long*  R1, int R2);

/**
 * The mod function returns the remainder division
 * The mod operator % in BETA appears to be broken
 * @param R0 is dividend or number to mod
 * @param R1 is divisor
 */
int mod(int R0, int R1);
